/*
 * Copyright (C) ILOG 2006. All rights reserved.
 * 
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OR ANY KIND
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MECHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */
package ilog.contrib.rules.bomdomainpopulate;

import ilog.contrib.rules.bomdomainpopulate.util.ILogger;
import ilog.contrib.rules.bomdomainpopulate.util.SystemErrLogger;
import ilog.rules.bom.IlrClass;
import ilog.rules.vocabulary.model.bom.IlrBOMDomainValueProvider;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * Implements IlrBOMDomainValueProvider. Uses connects to a JDBC datasource to
 * retrieve the item name, verbalization and translation of static references
 * defined in database.
 */
public class DatabaseDomainValueProvider implements IlrBOMDomainValueProvider {

	/*
	 * Internal representation of a domain value consisting of: name,
	 * verbalization and b2x.
	 */
	public static class Item {

		private final String name;

		private final String verbalization;

		private final String b2x;

		public Item(String name, String verbalization, String b2x) {
			this.name = name;
			this.verbalization = verbalization;
			this.b2x = b2x;
		}

		public String getB2x() {
			return b2x;
		}

		public String getName() {
			return name;
		}

		public String getVerbalization() {
			return verbalization;
		}
	}

	public static final String LOGGERIMPL = "logger.impl";

	public static final String DBDRIVER = "database.driver";

	public static final String DBURL = "database.url";

	public static final String DBUSER = "database.user";

	public static final String DBPASSWORD = "database.password";

	private static ILogger log = null;

	private static Properties props = null;

	private static DatabaseMappings dbMappings;

	// private Map<String, Item> items;
	private Map items;

	public DatabaseDomainValueProvider() {
		dbMappings = new DatabaseMappings();
		// items = new HashMap<String, Item>();
		items = new HashMap();
		ResourceBundle bundle = ResourceBundle.getBundle(getClass().getName());
		if (null == props) {
			props = new Properties();

			for (Enumeration keys = bundle.getKeys(); keys.hasMoreElements();) {
				final String key = (String) keys.nextElement();
				final String value = bundle.getString(key);

				props.put(key, value);
			}
		}
		log = initLogger();
		initJdbcDriver();
	}

	/*
	 * Get the ILogger. May return null if ILogger was not configured properly.
	 * @return ILogger or null if none configured.
	 */
	public ILogger getLogger() {
		return log;
	}

	public Collection getValues(IlrClass domainClass) {
		String className = domainClass.getFullyQualifiedName();
		loadItems(className);
		return items.keySet();
	}

	public String getBOM2XOMMapping(String valueName) {
		// Item item = items.get(valueName);
		Item item = (Item) items.get(valueName);
		if (null != item) {
			return item.getB2x();
		}
		return null;
	}

	public String getDisplayText(String valueName, Locale locale) {
		// Item item = items.get(valueName);
		Item item = (Item) items.get(valueName);
		if (null != item) {
			return item.getVerbalization();
		}
		return null;
	}

	/*
	 * return null since there is no specific documentation
	 */
	public String getDocumentation(String valueName, Locale locale) {
		return null;
	}

	/*
	 * return null since there is no specific property
	 */
	public Map getProperties(String valueName) {
		return null;
	}

	/*
	 * returns null. Translation is deprecated since JRules 6.0
	 */
	public String getTranslation(String valueName) {
		return null;
	}

	/*
	 * clear items cache
	 */
	public void dispose() {
		// do nothing.
	}

	/*
	 * Initialize the log, fallback to System.err when necessary
	 */
	private ILogger initLogger() {
		if (null == log) {
			String logImpl = props.getProperty(LOGGERIMPL);
			try {
				if (null != logImpl) {
					Class c = Class.forName(logImpl);
					log = (ILogger) c.newInstance();
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}

			// fallback to SystemErrLogger if log is not configured
			if (null == log) {
				log = SystemErrLogger.getInstance();
			}
		}
		return log;
	}

	/*
	 * Initializes the JDBC driver
	 */
	private void initJdbcDriver() {
		String jdbcDriver = props.getProperty(DBDRIVER);
		try {
			Class.forName(jdbcDriver);
		} catch (ClassNotFoundException e) {
			log.log("Unable to load JDBC driver.", e);
		}
	}

	/*
	 * Gets a database connection
	 */
	private Connection getConnection() {
		String url = props.getProperty(DBURL);
		String user = props.getProperty(DBUSER);
		String password = props.getProperty(DBPASSWORD);
		Connection conn = null;
		try {
			conn = DriverManager.getConnection(url, user, password);
		} catch (SQLException e) {
			log.log("Error getting JDBC connection", e);
		}
		return conn;
	}

	/*
	 * Load items for domain class from database.
	 */
	private void loadItems(String className) {
		
		Connection conn = getConnection();
		if (conn != null) {
			try {
				// retrieve table name for this particular class
				String tableName = dbMappings.getTableName(className);
			
				// retrieve column names of interest for the domains: name ,
				// verbalization and translation (b2x) columns
				String nameColumn = dbMappings.getItemColumnName(className);
				String verbalizationColumn = dbMappings
						.getVerbalizationColumnName(className);
				String b2xColumn = dbMappings.getB2XColumnName(className);

				// default query is very simple: adjust it here if necessary
				StringBuffer sb = new StringBuffer("select ");
				sb.append(nameColumn).append(", ").append(verbalizationColumn)
						.append(", ").append(b2xColumn).append(" from ")
						.append(tableName).append(" order by ")
						.append(nameColumn);
				Statement statement = conn.createStatement();
				ResultSet set = statement.executeQuery(sb.toString());
				// clear the previous values now that the query has been
				// correctly executed.
				items.clear();
				while (set.next()) {					
					String name = set.getString(nameColumn);				
					String verbalization = set.getString(verbalizationColumn);
					String b2x = set.getString(b2xColumn);
					items.put(name, new Item(name, verbalization, b2x));					
				}
			} catch (SQLException e) {
				log.log("Error loading data", e);
			}
		}
	}
}
